/*
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/***** Includes *****/
/* Standard C Libraries */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

/* TI Drivers */
#include <ti/drivers/rf/RF.h>
#include <ti/drivers/GPIO.h>

/* Driverlib Header files */
#include DeviceFamily_constructPath(driverlib/rf_prop_mailbox.h)

/* Board Header files */
#include "ti_drivers_config.h"

/* Application Header files */
#include "RFQueue.h"
#include <ti_radio_config.h>

#include <ti/display/Display.h>
#include <ti/display/DisplayUart2.h>
#include <ti/display/DisplayExt.h>
#include <ti/display/AnsiColor.h>


/***** Defines *****/

/* Packet RX Configuration */
#define DATA_ENTRY_HEADER_SIZE  8  /* Constant header size of a Generic Data Entry */
#define FIXED_LENGTH            30 /* Number of payload bytes to receive */
#define NUM_DATA_ENTRIES        2  /* NOTE: Only two data entries supported at the moment */
#define N                       6

/***** Prototypes *****/
static void callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e);

/***** Variable declarations *****/
static RF_Object rfObject;
static RF_Handle rfHandle;

static uint8_t
rxDataEntryBuffer[RF_QUEUE_DATA_ENTRY_BUFFER_SIZE(NUM_DATA_ENTRIES,
                                                  FIXED_LENGTH,
                                                  0)]
                                                  __attribute__((aligned(4)));

/* Receive dataQueue for RF Core to fill in data */
static dataQueue_t dataQueue;
static rfc_dataEntryGeneral_t* currentDataEntry;
static uint8_t packetLength;
static uint8_t* packetDataPointer;

uint8_t packet[FIXED_LENGTH];
static char display_buffer[N*FIXED_LENGTH] = {0};

/***** Function definitions *****/

void *mainThread(void *arg0)
{
    Display_init();

    Display_Params params;
    Display_Params_init(&params);
    params.lineClearMode = DISPLAY_CLEAR_BOTH;

    Display_Handle hSerial = Display_open(Display_Type_UART, &params);

    if (hSerial == NULL)
    {
        while (1);
    }

    RF_Params rfParams;
    RF_Params_init(&rfParams);

    GPIO_setConfig(CONFIG_GPIO_RLED, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
    GPIO_write(CONFIG_GPIO_RLED, CONFIG_GPIO_LED_OFF);

    if( RFQueue_defineQueue(&dataQueue,
                            rxDataEntryBuffer,
                            sizeof(rxDataEntryBuffer),
                            NUM_DATA_ENTRIES,
                            FIXED_LENGTH))
    {
        while(1);
    }

    RF_cmdPropRx.pQueue = &dataQueue;
    RF_cmdPropRx.maxPktLen = FIXED_LENGTH;
    RF_cmdPropRx.pktConf.bUseCrc = 0;

    rfHandle = RF_open(&rfObject, &RF_prop, (RF_RadioSetup*)&RF_cmdPropRadioDivSetup, &rfParams);

    RF_postCmd(rfHandle, (RF_Op*)&RF_cmdFs, RF_PriorityNormal, NULL, 0);

    Display_printf(hSerial, 0, 0, "30 Bytes of received wmbus packet (PHY = T-Mode):");
    Display_printf(hSerial, 0, 0, "\n\r");

    while(1)
    {
        RF_runCmd(rfHandle, (RF_Op*)&RF_cmdPropRx, RF_PriorityNormal, &callback, RF_EventRxEntryDone);

        uint8_t i = 0;
        uint16_t n = 0;

        uint8_t displayBufferTemp[N + 1];

        displayBufferTemp[N] = 0;

        for(i = 0; i < FIXED_LENGTH; i++)
        {
            sprintf(&display_buffer[n], "0x%02X, ", packet[i]);
            n += N;
        }

        for (i = 0; i <= FIXED_LENGTH; i++)
        {
            memcpy(displayBufferTemp, &display_buffer[i*N], N);
            Display_printf(hSerial, 0, 0, " %s", displayBufferTemp);
        }
        Display_printf(hSerial, 0, 0, "\n\r");
    }
}

void callback(RF_Handle h, RF_CmdHandle ch, RF_EventMask e)
{
    if (e & RF_EventRxEntryDone)
    {
        GPIO_toggle(CONFIG_GPIO_RLED);

        currentDataEntry = RFQueue_getDataEntry();

        packetDataPointer = (uint8_t*)(&currentDataEntry->data);

        memcpy(packet, packetDataPointer, FIXED_LENGTH);

        RFQueue_nextEntry();
    }
}
